#!/usr/bin/perl -w

$MIRROR_PAGE = "http://www.linux-mandrake.com/mirrors.list";
#$MIRROR_PAGE = "http://dre2/ftp.html";
$RC_FILE = "$ENV{HOME}/.mandrake-update";
$WGET = "wget --passive-ftp";
$LANG = $ENV{LC_ALL} || $ENV{LANGUAGE} || $ENV{LC_MESSAGES} || $ENV{LANG} || 'us';

$action = shift @ARGV || '';
if ($action eq 'save') {
    @lines = map { s/^(mirror:)/#$1/; $_ } `cat $RC_FILE 2>/dev/null`;
    open F, ">$RC_FILE";
    print F "mirror: ", <STDIN>;
    print F join '' => @lines;
    close F;
} elsif ($action eq 'mirrors_norc') {
    print_string_array(get_mirrors('norc'));
} elsif ($action eq 'descriptions') {
    print_string_array(get_descriptions(@ARGV));
} elsif ($action =~ /rpms/) {
    my $f = $action eq 'rpms' ? \&get_updates : \&get_updates2;
    my ($status, @list) = &$f(@ARGV);
    print_int($status);
    print_string_array(@list);
} else {
    print_string_array(get_mirrors());
}

sub get_updates {
    my $mirror = shift;
    my (@to_update, $rep, $name);
    if ($mirror =~ m|^/|) {
	# start with / => local
	open F, "find . -name '*.rpm' |";
	@to_update = map { chop; $_ } <F>;
    } else {
	open F, "$WGET $mirror/updates/$version/ls-lR -O - |";
	foreach (<F>) {
	    if (/^([^ ]+):$/) {
		$rep = "updates/$version/$1";
	    } elsif (/^-/) {
		$rep =~ /SRPMS/ and next;
		my @l = split;
		push @to_update, format_size($l[4]), "$rep/$l[8]" if $l[8];
	    }
	}
    }
    close F or return 0;

    get_updates2(@to_update)
}

sub get_updates2 {    
    # add ""/mandrake-release-(.*?)-/ and $version = $1;""  in the map to get the mandrake release number
    my @installed = map { chop; $_ } `rpm -qa`;

    my $installeds = join "|", map { my ($a) = /(.*)-.+?-/ or return 0; quotemeta($a) } @installed;
    my $installedf = join "|", map { quotemeta } @installed;

    my ($i, $size, @to_really_update);
    foreach (@_) {
	if ($i = !$i) {
	    $size = $_;
	} else {
	    m!(^|/)($installeds)-([^-]+)-[^-.]+\.[^.]+\.rpm! or next;
	    /$installedf/ and next;	# must not be exactly like already installed
	    push @to_really_update, $_, $2, $3, $size;
	}
    }
    1, @to_really_update
}

sub get_mirrors {
    my $option = shift || '';
    if (-r $RC_FILE && $option ne 'norc') {
	local *F;
	open F, $RC_FILE or die;
	foreach (<F>) {
	    /mirror: (.*)/ or next;
	    return $1, $1, "";
	}
	close F;
    }

    my @mirrors = map { chop; /^[fh]/ ? $_ : () } `$WGET $MIRROR_PAGE -O - 2>/dev/null`;
    @mirrors = map { ($a) = m,(?:ftp|http)://(.*?)/,; $a ? [ $_,$a, get_country($a) ] : () } @mirrors;
    map { @$_ } sort { $a->[2] cmp $b->[2] } @mirrors
}

sub get_descriptions {
    my $mirror = shift;
    my (%e, $text, $garb, @l);
    $mirror =~ m|^/| and return @l;

    my $save = sub {
	if ($e{package}) {
	    push @l, @e{qw(package noupgrade importance pre post description)};
	    %e = ();
	}
    };

    open F, "$WGET $mirror/updates/$version/descriptions -O - |";
    foreach (<F>) {
        if (/^%package(?:\s+-n)?\s+(.*)/i) {
	    &$save();
	    $text = undef;
	    $e{package} = $1;
	} elsif (/^%(\w+)(\s+-l\s+(.*))?/) {
	    $text = $3 eq $LANG || !$3 && !$e{$1} ? \$e{$1} : \$garb;
	} elsif ($text) {
	    $$text .= $_;
	} elsif (/^flag:\s+(.*)/) {
	    $e{noupgrade} = $1 =~ /install/i;
	} elsif (/^importance:\s+(.*)/) {
	    $e{importance} = $1;
	}
    }
    &$save();
    @l;
}

sub print_int($) { print pack("i", $_[0]); }
sub print_string { print_int(length($_[0]) + 1); print $_[0], "\0"; }
sub print_string_array { print_int(int @_); print_string(join "\0", @_); }

sub format_size {
    my ($s) = @_;
    my %l = (40 => 'T', 30 => 'G', 20 => 'M', 10 => 'K');

    foreach (sort { $b <=> $a } keys %l) {
	my $t = int ($s / (2 ** $_));
	return $t . $l{$_} if $t;
    }
    $s;
}

sub get_country {
    my ($a) = $_[0] =~ /\.([^.]+)$/;
    my $r = $country_codes{uc $a};
    $r ? join(' ', map { ucfirst(lc($_)) } split ' ', $r) : "USA?"
}

BEGIN {
%country_codes = ( # from ftp://ftp.ripe.net/iso3166-countrycodes
"AF", "AFGHANISTAN",
"AL", "ALBANIA",
"DZ", "ALGERIA",
"AS", "AMERICAN SAMOA",
"AD", "ANDORRA",
"AO", "ANGOLA",
"AI", "ANGUILLA",
"AQ", "ANTARCTICA",
"AG", "ANTIGUA AND BARBUDA",
"AR", "ARGENTINA",
"AM", "ARMENIA",
"AW", "ARUBA",
"AU", "AUSTRALIA",
"AT", "AUSTRIA",
"AZ", "AZERBAIJAN",
"BS", "BAHAMAS",
"BH", "BAHRAIN",
"BD", "BANGLADESH",
"BB", "BARBADOS",
"BY", "BELARUS",
"BE", "BELGIUM",
"BZ", "BELIZE",
"BJ", "BENIN",
"BM", "BERMUDA",
"BT", "BHUTAN",
"BO", "BOLIVIA",
"BA", "BOSNIA AND HERZEGOWINA",
"BW", "BOTSWANA",
"BV", "BOUVET ISLAND",
"BR", "BRAZIL",
"IO", "BRITISH INDIAN OCEAN TERRITORY",
"BN", "BRUNEI DARUSSALAM",
"BG", "BULGARIA",
"BF", "BURKINA FASO",
"BI", "BURUNDI",
"KH", "CAMBODIA",
"CM", "CAMEROON",
"CA", "CANADA",
"CV", "CAPE VERDE",
"KY", "CAYMAN ISLANDS",
"CF", "CENTRAL AFRICAN REPUBLIC",
"TD", "CHAD",
"CL", "CHILE",
"CN", "CHINA",
"CX", "CHRISTMAS ISLAND",
"CC", "COCOS (KEELING) ISLANDS",
"CO", "COLOMBIA",
"KM", "COMOROS",
"CG", "CONGO",
"CD", "CONGO, THE DEMOCRATIC REPUBLIC OF THE",
"CK", "COOK ISLANDS",
"CR", "COSTA RICA",
"CI", "COTE D'IVOIRE",
"HR", "CROATIA (local name: Hrvatska)",
"CU", "CUBA",
"CY", "CYPRUS",
"CZ", "CZECH REPUBLIC",
"DK", "DENMARK",
"DJ", "DJIBOUTI",
"DM", "DOMINICA",
"DO", "DOMINICAN REPUBLIC",
"TP", "EAST TIMOR",
"EC", "ECUADOR",
"EG", "EGYPT",
"SV", "EL SALVADOR",
"GQ", "EQUATORIAL GUINEA",
"ER", "ERITREA",
"EE", "ESTONIA",
"ET", "ETHIOPIA",
"FK", "FALKLAND ISLANDS (MALVINAS)",
"FO", "FAROE ISLANDS",
"FJ", "FIJI",
"FI", "FINLAND",
"FR", "FRANCE",
"FX", "FRANCE, METROPOLITAN",
"GF", "FRENCH GUIANA",
"PF", "FRENCH POLYNESIA",
"TF", "FRENCH SOUTHERN TERRITORIES",
"GA", "GABON",
"GM", "GAMBIA",
"GE", "GEORGIA",
"DE", "GERMANY",
"GH", "GHANA",
"GI", "GIBRALTAR",
"GR", "GREECE",
"GL", "GREENLAND",
"GD", "GRENADA",
"GP", "GUADELOUPE",
"GU", "GUAM",
"GT", "GUATEMALA",
"GN", "GUINEA",
"GW", "GUINEA-BISSAU",
"GY", "GUYANA",
"HT", "HAITI",
"HM", "HEARD AND MC DONALD ISLANDS",
"VA", "HOLY SEE (VATICAN CITY STATE)",
"HN", "HONDURAS",
"HK", "HONG KONG",
"HU", "HUNGARY",
"IS", "ICELAND",
"IN", "INDIA",
"ID", "INDONESIA",
"IR", "IRAN (ISLAMIC REPUBLIC OF)",
"IQ", "IRAQ",
"IE", "IRELAND",
"IL", "ISRAEL",
"IT", "ITALY",
"JM", "JAMAICA",
"JP", "JAPAN",
"JO", "JORDAN",
"KZ", "KAZAKHSTAN",
"KE", "KENYA",
"KI", "KIRIBATI",
"KP", "KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF",
"KR", "KOREA, REPUBLIC OF",
"KW", "KUWAIT",
"KG", "KYRGYZSTAN",
"LA", "LAO PEOPLE'S DEMOCRATIC REPUBLIC",
"LV", "LATVIA",
"LB", "LEBANON",
"LS", "LESOTHO",
"LR", "LIBERIA",
"LY", "LIBYAN ARAB JAMAHIRIYA",
"LI", "LIECHTENSTEIN",
"LT", "LITHUANIA",
"LU", "LUXEMBOURG",
"MO", "MACAU",
"MK", "MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF",
"MG", "MADAGASCAR",
"MW", "MALAWI",
"MY", "MALAYSIA",
"MV", "MALDIVES",
"ML", "MALI",
"MT", "MALTA",
"MH", "MARSHALL ISLANDS",
"MQ", "MARTINIQUE",
"MR", "MAURITANIA",
"MU", "MAURITIUS",
"YT", "MAYOTTE",
"MX", "MEXICO",
"FM", "MICRONESIA, FEDERATED STATES OF",
"MD", "MOLDOVA, REPUBLIC OF",
"MC", "MONACO",
"MN", "MONGOLIA",
"MS", "MONTSERRAT",
"MA", "MOROCCO",
"MZ", "MOZAMBIQUE",
"MM", "MYANMAR",
"NA", "NAMIBIA",
"NR", "NAURU",
"NP", "NEPAL",
"NL", "NETHERLANDS",
"AN", "NETHERLANDS ANTILLES",
"NC", "NEW CALEDONIA",
"NZ", "NEW ZEALAND",
"NI", "NICARAGUA",
"NE", "NIGER",
"NG", "NIGERIA",
"NU", "NIUE",
"NF", "NORFOLK ISLAND",
"MP", "NORTHERN MARIANA ISLANDS",
"NO", "NORWAY",
"OM", "OMAN",
"PK", "PAKISTAN",
"PW", "PALAU",
"PA", "PANAMA",
"PG", "PAPUA NEW GUINEA",
"PY", "PARAGUAY",
"PE", "PERU",
"PH", "PHILIPPINES",
"PN", "PITCAIRN",
"PL", "POLAND",
"PT", "PORTUGAL",
"PR", "PUERTO RICO",
"QA", "QATAR",
"RE", "REUNION",
"RO", "ROMANIA",
"RU", "RUSSIAN FEDERATION",
"RW", "RWANDA",
"KN", "SAINT KITTS AND NEVIS",
"LC", "SAINT LUCIA",
"VC", "SAINT VINCENT AND THE GRENADINES",
"WS", "SAMOA",
"SM", "SAN MARINO",
"ST", "SAO TOME AND PRINCIPE",
"SA", "SAUDI ARABIA",
"SN", "SENEGAL",
"SC", "SEYCHELLES",
"SL", "SIERRA LEONE",
"SG", "SINGAPORE",
"SK", "SLOVAKIA (Slovak Republic)",
"SI", "SLOVENIA",
"SB", "SOLOMON ISLANDS",
"SO", "SOMALIA",
"ZA", "SOUTH AFRICA",
"GS", "SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS",
"ES", "SPAIN",
"LK", "SRI LANKA",
"SH", "ST. HELENA",
"PM", "ST. PIERRE AND MIQUELON",
"SD", "SUDAN",
"SR", "SURINAME",
"SJ", "SVALBARD AND JAN MAYEN ISLANDS",
"SZ", "SWAZILAND",
"SE", "SWEDEN",
"CH", "SWITZERLAND",
"SY", "SYRIAN ARAB REPUBLIC",
"TW", "TAIWAN, PROVINCE OF CHINA",
"TJ", "TAJIKISTAN",
"TZ", "TANZANIA, UNITED REPUBLIC OF",
"TH", "THAILAND",
"TG", "TOGO",
"TK", "TOKELAU",
"TO", "TONGA",
"TT", "TRINIDAD AND TOBAGO",
"TN", "TUNISIA",
"TR", "TURKEY",
"TM", "TURKMENISTAN",
"TC", "TURKS AND CAICOS ISLANDS",
"TV", "TUVALU",
"UG", "UGANDA",
"UA", "UKRAINE",
"AE", "UNITED ARAB EMIRATES",
"GB", "UNITED KINGDOM",
"US", "UNITED STATES",
"UM", "UNITED STATES MINOR OUTLYING ISLANDS",
"UY", "URUGUAY",
"UZ", "UZBEKISTAN",
"VU", "VANUATU",
"VE", "VENEZUELA",
"VN", "VIET NAM",
"VG", "VIRGIN ISLANDS (BRITISH)",
"VI", "VIRGIN ISLANDS (U.S.)",
"WF", "WALLIS AND FUTUNA ISLANDS",
"EH", "WESTERN SAHARA",
"YE", "YEMEN",
"YU", "YUGOSLAVIA",
"ZM", "ZAMBIA",
"ZW", "ZIMBABWE",

#added
"UK", "UNITED KINGDOM",
);
}

BEGIN { $version = "7.0"; }
